@@ -42,6 +42,9 @@
                          (cell-error-name condition)
                          (type-of (unbound-slot-instance condition))))))))
 
+(define-condition missing-slot (cell-error simple-type-error)
+  ())
+
 ;;; These three functions work on std-instances and fsc-instances. These are
 ;;; instances for which it is possible to change the wrapper and the slots.
 ;;;
@@ -95,7 +98,7 @@
                  (return-from slot-value
                    (if cell
                        (funcall (slot-info-reader (cdr cell)) object)
-                       (values (slot-missing (wrapper-class* wrapper) object
+                       (values (slot-missing (wrapper-class wrapper) object
                                              slot-name 'slot-value)))))
                 ;; this next test means CONSP, but the transform that weakens
                 ;; CONSP to LISTP isn't working here for some reason.
@@ -104,14 +107,14 @@
                 (t
                  (bug "Bogus slot cell in SLOT-VALUE: ~S" cell)))))
     (if (unbound-marker-p value)
-        (slot-unbound (wrapper-class* wrapper) object slot-name)
+        (slot-unbound (wrapper-class wrapper) object slot-name)
         value)))
 
 (defun set-slot-value (object slot-name new-value)
   (let* ((wrapper (valid-wrapper-of object))
          (cell (or (find-slot-cell wrapper slot-name)
                    (return-from set-slot-value
-                     (progn (slot-missing (wrapper-class* wrapper)
+                     (progn (slot-missing (wrapper-class wrapper)
                                           object slot-name 'setf new-value)
                             new-value))))
          (location (car cell))
@@ -143,7 +146,7 @@
   (let* ((wrapper (valid-wrapper-of object))
          (cell (or (find-slot-cell wrapper slot-name)
                    (return-from slot-value
-                     (values (slot-missing (wrapper-class* wrapper) object slot-name
+                     (values (slot-missing (wrapper-class wrapper) object slot-name
                                            'cas (list old-value new-value))))))
          (location (car cell))
          (info (cdr cell))
@@ -163,7 +166,7 @@
                      (t
                       (bug "Bogus slot-cell in (CAS SLOT-VALUE): ~S" cell)))))
       (if (and (unbound-marker-p old) (neq old old-value))
-          (slot-unbound (wrapper-class* wrapper) object slot-name)
+          (slot-unbound (wrapper-class wrapper) object slot-name)
           old))))
 
 (defun slot-boundp (object slot-name)
@@ -179,7 +182,7 @@
                  (return-from slot-boundp
                    (if cell
                        (funcall (slot-info-boundp (cdr cell)) object)
-                       (and (slot-missing (wrapper-class* wrapper) object
+                       (and (slot-missing (wrapper-class wrapper) object
                                           slot-name 'slot-boundp)
                             t))))
                 ((listp location) ; forcibly transform CONSP to LISTP
@@ -199,10 +202,10 @@
                      +slot-unbound+)))
           ((not location)
            (if cell
-               (let ((class (wrapper-class* wrapper)))
+               (let ((class (wrapper-class wrapper)))
                  (slot-makunbound-using-class class object
                                               (find-slot-definition class slot-name)))
-               (slot-missing (wrapper-class* wrapper) object slot-name
+               (slot-missing (wrapper-class wrapper) object slot-name
                              'slot-makunbound)))
           ((listp location) ; forcibly transform CONSP to LISTP
            (setf (cdr location) +slot-unbound+))
@@ -369,26 +372,30 @@
 
 (defmethod slot-missing
            ((class t) instance slot-name operation &optional new-value)
-  (error "~@<When attempting to ~A, the slot ~S is missing from the ~
+  (error 'missing-slot
+         :name slot-name
+         :format-control
+         "~@<When attempting to ~A, the slot ~S is missing from the ~
           object ~S.~@[~%~a~]~@:>"
-         (ecase operation
-           (slot-value "read the slot's value (slot-value)")
-           (setf (format nil
-                         "set the slot's value to ~S (SETF of SLOT-VALUE)"
-                         new-value))
-           (slot-boundp "test to see whether slot is bound (SLOT-BOUNDP)")
-           (slot-makunbound "make the slot unbound (SLOT-MAKUNBOUND)"))
-         slot-name
-         instance
-         (let ((slot (and (typep class 'slot-class)
-                          (find slot-name (class-slots class)
-                                :key #'slot-definition-name
-                                :test #'string-equal))))
-           (when slot
-             (format nil "It has a slot ~/sb-ext:print-symbol-with-prefix/, while ~
+         :format-arguments
+         (list (ecase operation
+                 (slot-value "read the slot's value (slot-value)")
+                 (setf (format nil
+                               "set the slot's value to ~S (SETF of SLOT-VALUE)"
+                               new-value))
+                 (slot-boundp "test to see whether slot is bound (SLOT-BOUNDP)")
+                 (slot-makunbound "make the slot unbound (SLOT-MAKUNBOUND)"))
+               slot-name
+               instance
+               (let ((slot (and (typep class 'slot-class)
+                                (find slot-name (class-slots class)
+                                      :key #'slot-definition-name
+                                      :test #'string-equal))))
+                 (when slot
+                   (format nil "It has a slot ~/sb-ext:print-symbol-with-prefix/, while ~
                          ~/sb-ext:print-symbol-with-prefix/ is requested."
-                     (slot-definition-name slot)
-                     slot-name)))))
+                           (slot-definition-name slot)
+                           slot-name))))))
 
 (defmethod slot-unbound ((class t) instance slot-name)
   (restart-case
@@ -413,7 +420,7 @@
        ;; in list. The only exceptions are when there are non-local slots
        ;; before the one we want.
        (slot-definition-name
-        (find position (layout-slot-list (layout-of instance))
+        (find position (wrapper-slot-list (wrapper-of instance))
               :key #'slot-definition-location)))
       (cons
        (car position))))))
